@@ -280,7 +280,7 @@ namespace WTF {
     template<typename T, size_t inlineCapacity>
     void Deque<T, inlineCapacity>::checkIndexValidity(size_t index) const
     {
-        ASSERT(index <= m_buffer.capacity());
+        ASSERT_UNUSED(index, index <= m_buffer.capacity());
         if (m_start <= m_end) {
             ASSERT(index >= m_start);
             ASSERT(index <= m_end);
@@ -454,7 +454,7 @@ namespace WTF {
     {
         checkValidity();
         expandCapacityIfNeeded();
-        new (&m_buffer.buffer()[m_end]) T(value);
+        new (NotNull, &m_buffer.buffer()[m_end]) T(value);
         if (m_end == m_buffer.capacity() - 1)
             m_end = 0;
         else
@@ -471,7 +471,7 @@ namespace WTF {
             m_start = m_buffer.capacity() - 1;
         else
             --m_start;
-        new (&m_buffer.buffer()[m_start]) T(value);
+        new (NotNull, &m_buffer.buffer()[m_start]) T(value);
         checkValidity();
     }
 
