@@ -23,6 +23,7 @@
 #ifndef CSSParser_h
 #define CSSParser_h
 
+#include "CSSCalculationValue.h"
 #include "CSSGradientValue.h"
 #include "CSSParserValues.h"
 #include "CSSPropertySourceData.h"
@@ -31,15 +32,19 @@
 #include "MediaQuery.h"
 #include <wtf/HashMap.h>
 #include <wtf/HashSet.h>
+#include <wtf/OwnArrayPtr.h>
 #include <wtf/Vector.h>
 #include <wtf/text/AtomicString.h>
 
+#if ENABLE(CSS_FILTERS)
+#include "WebKitCSSFilterValue.h"
+#endif
+
 namespace WebCore {
 
 class CSSBorderImageSliceValue;
-class CSSMutableStyleDeclaration;
 class CSSPrimitiveValue;
-class CSSPrimitiveValueCache;
+class CSSValuePool;
 class CSSProperty;
 class CSSRule;
 class CSSRuleList;
@@ -47,13 +52,14 @@ class CSSSelectorList;
 class CSSStyleSheet;
 class CSSValue;
 class CSSValueList;
+class CSSWrapShape;
 class Document;
 class MediaList;
 class MediaQueryExp;
-class StyleBase;
+class StylePropertySet;
+class StyledElement;
 class WebKitCSSKeyframeRule;
 class WebKitCSSKeyframesRule;
-class CSSWrapShape;
 
 class CSSParser {
 public:
@@ -62,20 +68,19 @@ public:
 
     void parseSheet(CSSStyleSheet*, const String&, int startLineNumber = 0, StyleRuleRangeMap* ruleRangeMap = 0);
     PassRefPtr<CSSRule> parseRule(CSSStyleSheet*, const String&);
-    PassRefPtr<CSSRule> parseKeyframeRule(CSSStyleSheet*, const String&);
-    static bool parseValue(CSSMutableStyleDeclaration*, int propId, const String&, bool important, bool strict);
+    PassRefPtr<WebKitCSSKeyframeRule> parseKeyframeRule(CSSStyleSheet*, const String&);
+    static bool parseValue(StylePropertySet*, int propId, const String&, bool important, bool strict);
     static bool parseColor(RGBA32& color, const String&, bool strict = false);
     static bool parseSystemColor(RGBA32& color, const String&, Document*);
     PassRefPtr<CSSPrimitiveValue> parseValidPrimitive(int propId, CSSParserValue*);
-    bool parseColor(CSSMutableStyleDeclaration*, const String&);
-    bool parseDeclaration(CSSMutableStyleDeclaration*, const String&, RefPtr<CSSStyleSourceData>* styleSourceData = 0);
+    bool parseDeclaration(StylePropertySet*, const String&, RefPtr<CSSStyleSourceData>* = 0, CSSStyleSheet* contextStyleSheet = 0);
     bool parseMediaQuery(MediaList*, const String&);
 
-    Document* document() const;
+    Document* findDocument() const;
 
-    CSSPrimitiveValueCache* primitiveValueCache() const { return m_primitiveValueCache.get(); }
+    CSSValuePool* cssValuePool() const { return m_cssValuePool.get(); }
 
-    void addProperty(int propId, PassRefPtr<CSSValue>, bool important);
+    void addProperty(int propId, PassRefPtr<CSSValue>, bool important, bool implicit = false);
     void rollbackLastProperties(int num);
     bool hasProperties() const { return m_numParsedProperties > 0; }
 
@@ -89,14 +94,14 @@ public:
 
     PassRefPtr<CSSValue> parseBackgroundColor();
 
-    bool parseFillImage(RefPtr<CSSValue>&);
+    bool parseFillImage(CSSParserValueList*, RefPtr<CSSValue>&);
 
     enum FillPositionFlag { InvalidFillPosition = 0, AmbiguousFillPosition = 1, XFillPosition = 2, YFillPosition = 4 };
     PassRefPtr<CSSValue> parseFillPositionComponent(CSSParserValueList*, unsigned& cumulativeFlags, FillPositionFlag& individualFlag);
     PassRefPtr<CSSValue> parseFillPositionX(CSSParserValueList*);
     PassRefPtr<CSSValue> parseFillPositionY(CSSParserValueList*);
     void parseFillPosition(CSSParserValueList*, RefPtr<CSSValue>&, RefPtr<CSSValue>&);
-    
+
     void parseFillRepeat(RefPtr<CSSValue>&, RefPtr<CSSValue>&);
     PassRefPtr<CSSValue> parseFillSize(int propId, bool &allowComma);
 
@@ -123,11 +128,15 @@ public:
     bool parseTransitionShorthand(bool important);
     bool parseAnimationShorthand(bool important);
 
+#if ENABLE(CSS_GRID_LAYOUT)
+    bool parseGridTrackList(int propId, bool important);
+#endif
+
     bool parseDashboardRegions(int propId, bool important);
 
     bool parseShape(int propId, bool important);
 
-    bool parseWrapShape(bool important);
+    bool parseWrapShape(bool shapeInside, bool important);
     PassRefPtr<CSSWrapShape> parseWrapShapeRect(CSSParserValueList* args);
     PassRefPtr<CSSWrapShape> parseWrapShapeCircle(CSSParserValueList* args);
     PassRefPtr<CSSWrapShape> parseWrapShapeEllipse(CSSParserValueList* args);
@@ -145,9 +154,8 @@ public:
     bool parseColorFromValue(CSSParserValue*, RGBA32&);
     void parseSelector(const String&, Document* doc, CSSSelectorList&);
 
-    static bool parseColor(const String&, RGBA32& rgb, bool strict);
+    static bool fastParseColor(RGBA32&, const String&, bool strict);
 
-    bool parseFontStyle(bool important);
     bool parseFontVariant(bool important);
     bool parseFontWeight(bool important);
     bool parseFontFaceSrc();
@@ -160,31 +168,39 @@ public:
     PassRefPtr<CSSValue> parseSVGStrokeDasharray();
 #endif
 
-#if ENABLE(WCSS)
-    PassRefPtr<CSSValue> parseWCSSInputProperty();
-#endif
-
     // CSS3 Parsing Routines (for properties specific to CSS3)
-    bool parseShadow(int propId, bool important);
-    bool parseBorderImage(int propId, RefPtr<CSSValue>&);
+    PassRefPtr<CSSValueList> parseShadow(CSSParserValueList*, int propId);
+    bool parseBorderImage(int propId, RefPtr<CSSValue>&, bool important = false);
     bool parseBorderImageRepeat(RefPtr<CSSValue>&);
     bool parseBorderImageSlice(int propId, RefPtr<CSSBorderImageSliceValue>&);
     bool parseBorderImageWidth(RefPtr<CSSPrimitiveValue>&);
     bool parseBorderImageOutset(RefPtr<CSSPrimitiveValue>&);
     bool parseBorderRadius(int propId, bool important);
 
+    bool parseAspectRatio(bool important);
+
     bool parseReflect(int propId, bool important);
 
     bool parseFlex(int propId, bool important);
 
     // Image generators
-    bool parseCanvas(RefPtr<CSSValue>&);
+    bool parseCanvas(CSSParserValueList*, RefPtr<CSSValue>&);
 
-    bool parseDeprecatedGradient(RefPtr<CSSValue>&);
-    bool parseLinearGradient(RefPtr<CSSValue>&, CSSGradientRepeat repeating);
-    bool parseRadialGradient(RefPtr<CSSValue>&, CSSGradientRepeat repeating);
+    bool parseDeprecatedGradient(CSSParserValueList*, RefPtr<CSSValue>&);
+    bool parseLinearGradient(CSSParserValueList*, RefPtr<CSSValue>&, CSSGradientRepeat repeating);
+    bool parseRadialGradient(CSSParserValueList*, RefPtr<CSSValue>&, CSSGradientRepeat repeating);
     bool parseGradientColorStops(CSSParserValueList*, CSSGradientValue*, bool expectComma);
 
+    bool parseCrossfade(CSSParserValueList*, RefPtr<CSSValue>&);
+
+#if ENABLE(CSS_FILTERS)
+    PassRefPtr<CSSValueList> parseFilter();
+    PassRefPtr<WebKitCSSFilterValue> parseBuiltinFilterArguments(CSSParserValueList*, WebKitCSSFilterValue::FilterOperationType);
+#if ENABLE(CSS_SHADERS)
+    PassRefPtr<WebKitCSSFilterValue> parseCustomFilter(CSSParserValue*);
+#endif
+#endif
+
     PassRefPtr<CSSValueList> parseTransform();
     bool parseTransformOrigin(int propId, int& propId1, int& propId2, int& propId3, RefPtr<CSSValue>&, RefPtr<CSSValue>&, RefPtr<CSSValue>&);
     bool parsePerspectiveOrigin(int propId, int& propId1, int& propId2,  RefPtr<CSSValue>&, RefPtr<CSSValue>&);
@@ -192,12 +208,16 @@ public:
     bool parseTextEmphasisStyle(bool important);
 
     bool parseLineBoxContain(bool important);
+    bool parseCalculation(CSSParserValue*);
 
     bool parseFontFeatureTag(CSSValueList*);
     bool parseFontFeatureSettings(bool important);
 
+    bool cssRegionsEnabled() const;
     bool parseFlowThread(int propId, bool important);
-    PassRefPtr<CSSValue> parseFromFlowContent(CSSParserValueList*);
+    bool parseRegionThread(int propId, bool important);
+
+    bool parseFontVariantLigatures(bool important);
 
     int yyparse();
 
@@ -208,10 +228,10 @@ public:
     PassOwnPtr<Vector<OwnPtr<CSSParserSelector> > > sinkFloatingSelectorVector(Vector<OwnPtr<CSSParserSelector> >*);
 
     CSSParserValueList* createFloatingValueList();
-    CSSParserValueList* sinkFloatingValueList(CSSParserValueList*);
+    PassOwnPtr<CSSParserValueList> sinkFloatingValueList(CSSParserValueList*);
 
     CSSParserFunction* createFloatingFunction();
-    CSSParserFunction* sinkFloatingFunction(CSSParserFunction*);
+    PassOwnPtr<CSSParserFunction> sinkFloatingFunction(CSSParserFunction*);
 
     CSSParserValue& sinkFloatingValue(CSSParserValue&);
 
@@ -225,6 +245,7 @@ public:
     CSSRule* createStyleRule(Vector<OwnPtr<CSSParserSelector> >* selectors);
     CSSRule* createFontFaceRule();
     CSSRule* createPageRule(PassOwnPtr<CSSParserSelector> pageSelector);
+    CSSRule* createRegionRule(Vector<OwnPtr<CSSParserSelector> >* regionSelector, CSSRuleList* rules);
     CSSRule* createMarginAtRule(CSSSelector::MarginBoxType marginBox);
     void startDeclarationsForMarginBox();
     void endDeclarationsForMarginBox();
@@ -245,7 +266,11 @@ public:
 
     Vector<OwnPtr<CSSParserSelector> >* reusableSelectorVector() { return &m_reusableSelectorVector; }
 
+    void setReusableRegionSelectorVector(Vector<OwnPtr<CSSParserSelector> >* selectors);
+    Vector<OwnPtr<CSSParserSelector> >* reusableRegionSelectorVector() { return &m_reusableRegionSelectorVector; }
+
     void updateLastSelectorLineAndPosition();
+    void updateLastMediaLine(MediaList*);
 
     void clearProperties();
 
@@ -254,12 +279,13 @@ public:
     int m_id;
     CSSStyleSheet* m_styleSheet;
     RefPtr<CSSRule> m_rule;
-    RefPtr<CSSRule> m_keyframe;
+    RefPtr<WebKitCSSKeyframeRule> m_keyframe;
     OwnPtr<MediaQuery> m_mediaQuery;
-    CSSParserValueList* m_valueList;
+    OwnPtr<CSSParserValueList> m_valueList;
     CSSProperty** m_parsedProperties;
     CSSSelectorList* m_selectorListForParseSelector;
-    RefPtr<CSSPrimitiveValueCache> m_primitiveValueCache;
+
+    RefPtr<CSSValuePool> m_cssValuePool;
     unsigned m_numParsedProperties;
     unsigned m_maxParsedProperties;
     unsigned m_numParsedPropertiesBeforeMarginBox;
@@ -290,14 +316,32 @@ public:
     void resetRuleBodyMarks() { m_ruleBodyRange.start = m_ruleBodyRange.end = 0; }
     void resetPropertyMarks() { m_propertyRange.start = m_propertyRange.end = UINT_MAX; }
     int lex(void* yylval);
-    int token() { return yyTok; }
-    UChar* text(int* length);
-    void countLines();
-    int lex();
+    int token() { return m_token; }
 
+    PassRefPtr<CSSPrimitiveValue> createPrimitiveNumericValue(CSSParserValue*);
+    PassRefPtr<CSSPrimitiveValue> createPrimitiveStringValue(CSSParserValue*);
+        
 private:
+    inline bool isIdentifierStart();
+
+    static inline UChar* checkAndSkipString(UChar*, UChar);
+
+    void parseEscape(UChar*&);
+    inline void parseIdentifier(UChar*&, bool&);
+    inline void parseString(UChar*&, UChar);
+    inline void parseURI(UChar*&, UChar*&);
+    inline bool parseUnicodeRange();
+    bool parseNthChild();
+    bool parseNthChildExtra();
+    inline void detectFunctionTypeToken(int);
+    inline void detectMediaQueryToken(int);
+    inline void detectNumberToken(UChar*, int);
+    inline void detectDashToken(int);
+    inline void detectAtToken(int, bool);
+
     void setStyleSheet(CSSStyleSheet*);
-    
+    void ensureCSSValuePool();
+
     void recheckAtKeyword(const UChar* str, int len);
 
     void setupParser(const char* prefix, const String&, const char* suffix);
@@ -309,9 +353,9 @@ private:
     void deleteFontFaceOnlyValues();
 
     bool isGeneratedImageValue(CSSParserValue*) const;
-    bool parseGeneratedImage(RefPtr<CSSValue>&);
+    bool parseGeneratedImage(CSSParserValueList*, RefPtr<CSSValue>&);
 
-    bool parseValue(CSSMutableStyleDeclaration*, int propId, const String&, bool important);
+    bool parseValue(StylePropertySet*, int propId, const String&, bool important, CSSStyleSheet* contextStyleSheet = 0);
 
     enum SizeParameterType {
         None,
@@ -325,22 +369,30 @@ private:
     bool parseSize(int propId, bool important);
     SizeParameterType parseSizeParameter(CSSValueList* parsedValues, CSSParserValue* value, SizeParameterType prevParamType);
 
-    UChar* m_data;
-    UChar* yytext;
-    UChar* yy_c_buf_p;
-    UChar yy_hold_char;
-    int yy_last_accepting_state;
-    UChar* yy_last_accepting_cpos;
-    int yyleng;
-    int yyTok;
-    int yy_start;
+    bool parseFontFaceSrcURI(CSSValueList*);
+    bool parseFontFaceSrcLocal(CSSValueList*);
+
+    bool parseColor(const String&);
+
+    enum ParsingMode {
+        NormalMode,
+        MediaQueryMode,
+        NthChildMode
+    };
+
+    ParsingMode m_parsingMode;
+    OwnArrayPtr<UChar> m_dataStart;
+    UChar* m_currentCharacter;
+    UChar* m_tokenStart;
+    int m_token;
     int m_lineNumber;
     int m_lastSelectorLineNumber;
 
     bool m_allowImportRules;
     bool m_allowNamespaceDeclarations;
 
-    Vector<RefPtr<StyleBase> > m_parsedStyleObjects;
+    Vector<RefPtr<CSSRule> > m_parsedRules;
+    Vector<RefPtr<MediaList> > m_parsedMediaLists;
     Vector<RefPtr<CSSRuleList> > m_parsedRuleLists;
     HashSet<CSSParserSelector*> m_floatingSelectors;
     HashSet<Vector<OwnPtr<CSSParserSelector> >*> m_floatingSelectorVectors;
@@ -352,6 +404,9 @@ private:
     OwnPtr<Vector<OwnPtr<MediaQueryExp> > > m_floatingMediaQueryExpList;
 
     Vector<OwnPtr<CSSParserSelector> > m_reusableSelectorVector;
+    Vector<OwnPtr<CSSParserSelector> > m_reusableRegionSelectorVector;
+
+    RefPtr<CSSCalcValue> m_parsedCalculation;
 
     // defines units allowed for a certain property, used in parseUnit
     enum Units {
@@ -372,14 +427,23 @@ private:
         return static_cast<Units>(static_cast<unsigned>(a) | static_cast<unsigned>(b));
     }
 
-    static bool validUnit(CSSParserValue*, Units, bool strict);
+    bool validCalculationUnit(CSSParserValue*, Units);
+    bool validUnit(CSSParserValue*, Units, bool strict);
 
     bool parseBorderImageQuad(Units, RefPtr<CSSPrimitiveValue>&);
-    
-    PassRefPtr<CSSPrimitiveValue> createPrimitiveNumericValue(CSSParserValue*);
-    PassRefPtr<CSSPrimitiveValue> createPrimitiveStringValue(CSSParserValue*);
+    int colorIntFromValue(CSSParserValue*);
 
+    enum ReleaseParsedCalcValueCondition {
+        ReleaseParsedCalcValue,
+        DoNotReleaseParsedCalcValue
+    };    
+    double parsedDouble(CSSParserValue*, ReleaseParsedCalcValueCondition releaseCalc = DoNotReleaseParsedCalcValue);
+    bool isCalculation(CSSParserValue*);
+    
     friend class TransformOperationInfo;
+#if ENABLE(CSS_FILTERS)
+    friend class FilterOperationInfo;
+#endif
 };
 
 int cssPropertyID(const CSSParserString&);
