@@ -43,7 +43,6 @@
 #include "HTMLInputElement.h"
 #include "HiddenInputType.h"
 #include "ImageInputType.h"
-#include "IsIndexInputType.h"
 #include "KeyboardEvent.h"
 #include "LocalizedStrings.h"
 #include "MonthInputType.h"
@@ -57,6 +56,7 @@
 #include "ResetInputType.h"
 #include "SearchInputType.h"
 #include "ShadowRoot.h"
+#include "ShadowRootList.h"
 #include "SubmitInputType.h"
 #include "TelephoneInputType.h"
 #include "TextInputType.h"
@@ -83,15 +83,22 @@ static PassOwnPtr<InputTypeFactoryMap> createInputTypeFactoryMap()
 #if ENABLE(INPUT_COLOR)
     map->add(InputTypeNames::color(), ColorInputType::create);
 #endif
+#if ENABLE(INPUT_TYPE_DATE)
     map->add(InputTypeNames::date(), DateInputType::create);
+#endif
+#if ENABLE(INPUT_TYPE_DATETIME)
     map->add(InputTypeNames::datetime(), DateTimeInputType::create);
+#endif
+#if ENABLE(INPUT_TYPE_DATETIMELOCAL)
     map->add(InputTypeNames::datetimelocal(), DateTimeLocalInputType::create);
+#endif
     map->add(InputTypeNames::email(), EmailInputType::create);
     map->add(InputTypeNames::file(), FileInputType::create);
     map->add(InputTypeNames::hidden(), HiddenInputType::create);
     map->add(InputTypeNames::image(), ImageInputType::create);
-    map->add(InputTypeNames::isindex(), IsIndexInputType::create);
+#if ENABLE(INPUT_TYPE_MONTH)
     map->add(InputTypeNames::month(), MonthInputType::create);
+#endif
     map->add(InputTypeNames::number(), NumberInputType::create);
     map->add(InputTypeNames::password(), PasswordInputType::create);
     map->add(InputTypeNames::radio(), RadioInputType::create);
@@ -100,9 +107,13 @@ static PassOwnPtr<InputTypeFactoryMap> createInputTypeFactoryMap()
     map->add(InputTypeNames::search(), SearchInputType::create);
     map->add(InputTypeNames::submit(), SubmitInputType::create);
     map->add(InputTypeNames::telephone(), TelephoneInputType::create);
+#if ENABLE(INPUT_TYPE_TIME)
     map->add(InputTypeNames::time(), TimeInputType::create);
+#endif
     map->add(InputTypeNames::url(), URLInputType::create);
+#if ENABLE(INPUT_TYPE_WEEK)
     map->add(InputTypeNames::week(), WeekInputType::create);
+#endif
     // No need to register "text" because it is the default type.
     return map.release();
 }
@@ -182,7 +193,7 @@ double InputType::valueAsNumber() const
     return numeric_limits<double>::quiet_NaN();
 }
 
-void InputType::setValueAsNumber(double, bool, ExceptionCode& ec) const
+void InputType::setValueAsNumber(double, TextFieldEventBehavior, ExceptionCode& ec) const
 {
     ec = INVALID_STATE_ERR;
 }
@@ -368,7 +379,11 @@ void InputType::createShadowSubtree()
 
 void InputType::destroyShadowSubtree()
 {
-    element()->removeShadowRoot();
+    if (!element()->hasShadowRoot())
+        return;
+
+    if (ShadowRoot* root = element()->shadowRootList()->oldestShadowRoot())
+        root->removeAllChildren();
 }
 
 double InputType::parseToDouble(const String&, double defaultValue) const
@@ -424,7 +439,11 @@ bool InputType::shouldUseInputMethod() const
     return false;
 }
 
-void InputType::willBlur()
+void InputType::handleFocusEvent()
+{
+}
+
+void InputType::handleBlurEvent()
 {
 }
 
@@ -496,12 +515,12 @@ bool InputType::getTypeSpecificValue(String&)
     return false;
 }
 
-String InputType::fallbackValue()
+String InputType::fallbackValue() const
 {
     return String();
 }
 
-String InputType::defaultValue()
+String InputType::defaultValue() const
 {
     return String();
 }
@@ -521,15 +540,12 @@ bool InputType::storesValueSeparateFromAttribute()
     return true;
 }
 
-void InputType::setValue(const String& sanitizedValue, bool, bool sendChangeEvent)
+void InputType::setValue(const String& sanitizedValue, bool valueChanged, TextFieldEventBehavior eventBehavior)
 {
-    element()->setValueInternal(sanitizedValue, sendChangeEvent);
+    element()->setValueInternal(sanitizedValue, eventBehavior);
     element()->setNeedsStyleRecalc();
-}
-
-void InputType::dispatchChangeEventInResponseToSetValue()
-{
-    element()->dispatchFormControlChangeEvent();
+    if (valueChanged && eventBehavior != DispatchNoEvent)
+        element()->dispatchFormControlChangeEvent();
 }
 
 bool InputType::canSetValue(const String&)
@@ -561,7 +577,7 @@ bool InputType::isAcceptableValue(const String&)
     return true;
 }
 
-String InputType::sanitizeValue(const String& proposedValue)
+String InputType::sanitizeValue(const String& proposedValue) const
 {
     return proposedValue;
 }
@@ -710,6 +726,16 @@ void InputType::readonlyAttributeChanged()
 {
 }
 
+String InputType::defaultToolTip() const
+{
+    return String();
+}
+
+bool InputType::supportsIndeterminateAppearance() const
+{
+    return false;
+}
+
 namespace InputTypeNames {
 
 // The type names must be lowercased because they will be the return values of
@@ -777,12 +803,6 @@ const AtomicString& image()
     return name;
 }
 
-const AtomicString& isindex()
-{
-    DEFINE_STATIC_LOCAL(AtomicString, name, ("khtml_isindex"));
-    return name;
-}
-
 const AtomicString& month()
 {
     DEFINE_STATIC_LOCAL(AtomicString, name, ("month"));
