@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2007 Apple Inc. All rights reserved.
+ * Copyright (C) 2007, 2012 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -56,6 +56,9 @@ KeyframeAnimation::KeyframeAnimation(const Animation* animation, RenderObject* r
 
     // Update the m_transformFunctionListValid flag based on whether the function lists in the keyframes match.
     validateTransformFunctionList();
+#if ENABLE(CSS_FILTERS)
+    checkForMatchingFilterFunctionLists();
+#endif
 }
 
 KeyframeAnimation::~KeyframeAnimation()
@@ -85,25 +88,7 @@ void KeyframeAnimation::fetchIntervalEndpointsForProperty(int property, const Re
     if (m_animation->duration() && m_animation->iterationCount() != Animation::IterationCountInfinite)
         elapsedTime = min(elapsedTime, m_animation->duration() * m_animation->iterationCount());
 
-    double fractionalTime = m_animation->duration() ? (elapsedTime / m_animation->duration()) : 1;
-
-    // FIXME: startTime can be before the current animation "frame" time. This is to sync with the frame time
-    // concept in AnimationTimeController. So we need to somehow sync the two. Until then, the possible
-    // error is small and will probably not be noticeable. Until we fix this, remove the assert.
-    // https://bugs.webkit.org/show_bug.cgi?id=52037
-    // ASSERT(fractionalTime >= 0);
-    if (fractionalTime < 0)
-        fractionalTime = 0;
-
-    // FIXME: share this code with AnimationBase::progress().
-    int iteration = static_cast<int>(fractionalTime);
-    if (m_animation->iterationCount() != Animation::IterationCountInfinite)
-        iteration = min(iteration, m_animation->iterationCount() - 1);
-    fractionalTime -= iteration;
-    
-    bool reversing = (m_animation->direction() == Animation::AnimationDirectionAlternate) && (iteration & 1);
-    if (reversing)
-        fractionalTime = 1 - fractionalTime;
+    const double fractionalTime = this->fractionalTime(1, elapsedTime, 0);
 
     size_t numKeyframes = m_keyframes.size();
     if (!numKeyframes)
@@ -246,14 +231,7 @@ void KeyframeAnimation::getAnimatedStyle(RefPtr<RenderStyle>& animatedStyle)
 
 bool KeyframeAnimation::hasAnimationForProperty(int property) const
 {
-    // FIXME: why not just m_keyframes.containsProperty()?
-    HashSet<int>::const_iterator end = m_keyframes.endProperties();
-    for (HashSet<int>::const_iterator it = m_keyframes.beginProperties(); it != end; ++it) {
-        if (*it == property)
-            return true;
-    }
-    
-    return false;
+    return m_keyframes.containsProperty(property);
 }
 
 bool KeyframeAnimation::startAnimation(double timeOffset)
@@ -386,12 +364,7 @@ void KeyframeAnimation::resumeOverriddenAnimations()
 
 bool KeyframeAnimation::affectsProperty(int property) const
 {
-    HashSet<int>::const_iterator end = m_keyframes.endProperties();
-    for (HashSet<int>::const_iterator it = m_keyframes.beginProperties(); it != end; ++it) {
-        if (*it == property)
-            return true;
-    }
-    return false;
+    return m_keyframes.containsProperty(property);
 }
 
 void KeyframeAnimation::validateTransformFunctionList()
@@ -423,25 +396,59 @@ void KeyframeAnimation::validateTransformFunctionList()
         const KeyframeValue& currentKeyframe = m_keyframes[i];
         const TransformOperations* val = &currentKeyframe.style()->transform();
         
-        // A null transform matches anything
+        // An emtpy transform list matches anything.
         if (val->operations().isEmpty())
             continue;
         
-        // If the sizes of the function lists don't match, the lists don't match
-        if (firstVal->operations().size() != val->operations().size())
+        if (!firstVal->operationsMatch(*val))
             return;
-        
-        // If the types of each function are not the same, the lists don't match
-        for (size_t j = 0; j < firstVal->operations().size(); ++j) {
-            if (!firstVal->operations()[j]->isSameType(*val->operations()[j]))
-                return;
-        }
     }
 
     // Keyframes are valid
     m_transformFunctionListValid = true;
 }
 
+#if ENABLE(CSS_FILTERS)
+void KeyframeAnimation::checkForMatchingFilterFunctionLists()
+{
+    m_filterFunctionListsMatch = false;
+
+    if (m_keyframes.size() < 2 || !m_keyframes.containsProperty(CSSPropertyWebkitFilter))
+        return;
+
+    // Empty filters match anything, so find the first non-empty entry as the reference
+    size_t numKeyframes = m_keyframes.size();
+    size_t firstNonEmptyFilterKeyframeIndex = numKeyframes;
+
+    for (size_t i = 0; i < numKeyframes; ++i) {
+        const KeyframeValue& currentKeyframe = m_keyframes[i];
+        if (currentKeyframe.style()->filter().operations().size()) {
+            firstNonEmptyFilterKeyframeIndex = i;
+            break;
+        }
+    }
+    
+    if (firstNonEmptyFilterKeyframeIndex == numKeyframes)
+        return;
+        
+    const FilterOperations* firstVal = &m_keyframes[firstNonEmptyFilterKeyframeIndex].style()->filter();
+    
+    for (size_t i = firstNonEmptyFilterKeyframeIndex + 1; i < numKeyframes; ++i) {
+        const KeyframeValue& currentKeyframe = m_keyframes[i];
+        const FilterOperations* val = &currentKeyframe.style()->filter();
+        
+        // An emtpy filter list matches anything.
+        if (val->operations().isEmpty())
+            continue;
+        
+        if (!firstVal->operationsMatch(*val))
+            return;
+    }
+    
+    m_filterFunctionListsMatch = true;
+}
+#endif
+
 double KeyframeAnimation::timeToNextService()
 {
     double t = AnimationBase::timeToNextService();
