@@ -38,6 +38,7 @@
 #include "MediaPlayerPrivate.h"
 #include "Settings.h"
 #include "TimeRanges.h"
+#include <wtf/text/CString.h>
 
 #if PLATFORM(QT)
 #include <QtGlobal>
@@ -330,19 +331,21 @@ MediaPlayer::~MediaPlayer()
     m_mediaPlayerClient = 0;
 }
 
-bool MediaPlayer::load(const String& url, const ContentType& contentType)
+bool MediaPlayer::load(const KURL& url, const ContentType& contentType)
 {
     String type = contentType.type().lower();
     String typeCodecs = contentType.parameter(codecs());
+    String urlString = url.string();
 
     // If the MIME type is missing or is not meaningful, try to figure it out from the URL.
     if (type.isEmpty() || type == applicationOctetStream() || type == textPlain()) {
-        if (protocolIs(url, "data"))
-            type = mimeTypeFromDataURL(url);
+        if (protocolIs(urlString, "data"))
+            type = mimeTypeFromDataURL(urlString);
         else {
-            size_t pos = url.reverseFind('.');
+            String lastPathComponent = url.lastPathComponent();
+            size_t pos = lastPathComponent.reverseFind('.');
             if (pos != notFound) {
-                String extension = url.substring(pos + 1);
+                String extension = lastPathComponent.substring(pos + 1);
                 String mediaType = MIMETypeRegistry::getMediaMIMETypeForExtension(extension);
                 if (!mediaType.isEmpty())
                     type = mediaType;
@@ -350,7 +353,7 @@ bool MediaPlayer::load(const String& url, const ContentType& contentType)
         }
     }
 
-    m_url = url;
+    m_url = urlString;
     m_contentMIMEType = type;
     m_contentTypeCodecs = typeCodecs;
     loadWithNextMediaEngine(0);
@@ -391,8 +394,10 @@ void MediaPlayer::loadWithNextMediaEngine(MediaPlayerFactory* current)
         m_private->load(m_url);
     else {
         m_private = createNullMediaPlayer(this);
-        if (m_mediaPlayerClient)
+        if (m_mediaPlayerClient) {
             m_mediaPlayerClient->mediaPlayerEngineUpdated(this);
+            m_mediaPlayerClient->mediaPlayerResourceNotSupported(this);
+        }
     }
 }    
 
@@ -494,6 +499,11 @@ bool MediaPlayer::supportsSave() const
     return m_private->supportsSave();
 }
 
+bool MediaPlayer::supportsScanning() const
+{
+    return m_private->supportsScanning();
+}
+
 IntSize MediaPlayer::naturalSize()
 {
     return m_private->naturalSize();
@@ -604,6 +614,11 @@ PassRefPtr<TimeRanges> MediaPlayer::buffered()
     return m_private->buffered();
 }
 
+PassRefPtr<TimeRanges> MediaPlayer::seekable()
+{
+    return m_private->seekable();
+}
+
 float MediaPlayer::maxTimeSeekable()
 {
     return m_private->maxTimeSeekable();
