@@ -71,6 +71,7 @@
 #include <gtk/gtk.h>
 
 #if defined(XP_UNIX)
+#define String XtStringType
 #include "RefPtrCairo.h"
 #include "gtk2xtbin.h"
 #define Bool int // this got undefined somewhere
@@ -81,6 +82,7 @@
 #elif defined(GDK_WINDOWING_WIN32)
 #include "PluginMessageThrottlerWin.h"
 #include <gdk/gdkwin32.h>
+#undef String
 #endif
 
 using JSC::ExecState;
@@ -163,6 +165,8 @@ void PluginView::updatePluginWidget()
 void PluginView::setFocus(bool focused)
 {
     ASSERT(platformPluginWidget() == platformWidget());
+    if (focused && platformWidget())
+        gtk_widget_grab_focus(platformWidget());
     Widget::setFocus(focused);
 }
 
@@ -507,9 +511,9 @@ void PluginView::setNPWindowIfNeeded()
     if (m_isWindowed && !platformPluginWidget())
         return;
 
-    // If width or height are null, set the clipRect to null, indicating that
-    // the plugin is not visible/scrolled out.
-    if (!m_clipRect.isEmpty()) {
+    if (m_clipRect.isEmpty()) {
+        // If width or height are null, set the clipRect to null,
+        // indicating that the plugin is not visible/scrolled out.
         m_npWindow.clipRect.left = 0;
         m_npWindow.clipRect.right = 0;
         m_npWindow.clipRect.top = 0;
@@ -574,9 +578,23 @@ void PluginView::updateWidgetAllocationAndClip()
 #endif
     }
 
-    GtkAllocation allocation(m_delayedAllocation);
-    gtk_widget_size_allocate(widget, &allocation);
-    m_delayedAllocation = IntRect();
+    // The goal is to try to avoid calling gtk_widget_size_allocate in the WebView's
+    // size-allocate method. It blocks the main loop and if the widget is offscreen
+    // or hasn't moved it isn't required.
+
+    // Don't do anything if the allocation has not changed.
+    GtkAllocation currentAllocation;
+    gtk_widget_get_allocation(widget, &currentAllocation);
+    if (currentAllocation == m_delayedAllocation)
+        return;
+
+    // Don't do anything if both the old and the new allocations are outside the frame.
+    IntRect currentAllocationRect(currentAllocation);
+    currentAllocationRect.intersect(frameRect());
+    if (currentAllocationRect.isEmpty() && m_clipRect.isEmpty())
+        return;
+
+    g_object_set_data(G_OBJECT(widget), "delayed-allocation", &m_delayedAllocation);
 }
 
 void PluginView::setParentVisible(bool visible)
@@ -865,8 +883,12 @@ bool PluginView::platformStart()
             g_signal_connect(platformPluginWidget(), "plug-removed", G_CALLBACK(PluginView::plugRemovedCallback), this);
         } else
             setPlatformWidget(gtk_xtbin_new(pageClient, 0));
+#else
+#if OS(WINDOWS) && !defined(GTK_API_VERSION_2)
+        setPlatformWidget(0);
 #else
         setPlatformWidget(gtk_socket_new());
+#endif
         gtk_container_add(GTK_CONTAINER(pageClient), platformPluginWidget());
 #endif
     } else {
@@ -889,14 +911,14 @@ bool PluginView::platformStart()
         if (m_needsXEmbed) {
             GtkWidget* widget = platformPluginWidget();
             gtk_widget_realize(widget);
-            m_npWindow.window = (void*)gtk_socket_get_id(GTK_SOCKET(platformPluginWidget()));
+            m_npWindow.window = reinterpret_cast<void*>(gtk_socket_get_id(GTK_SOCKET(platformPluginWidget())));
             GdkWindow* window = gtk_widget_get_window(widget);
             ws->display = GDK_WINDOW_XDISPLAY(window);
             ws->visual = GDK_VISUAL_XVISUAL(gdk_window_get_visual(window));
             ws->depth = gdk_visual_get_depth(gdk_window_get_visual(window));
             ws->colormap = XCreateColormap(ws->display, GDK_ROOT_WINDOW(), ws->visual, AllocNone);
         } else {
-            m_npWindow.window = (void*)GTK_XTBIN(platformPluginWidget())->xtwindow;
+            m_npWindow.window = reinterpret_cast<void*>((GTK_XTBIN(platformPluginWidget())->xtwindow));
             ws->display = GTK_XTBIN(platformPluginWidget())->xtdisplay;
             ws->visual = GTK_XTBIN(platformPluginWidget())->xtclient.xtvisual;
             ws->depth = GTK_XTBIN(platformPluginWidget())->xtclient.xtdepth;
